mod sync_dir;

use std::{env, fs, io, thread, time};
use std::collections::HashMap;
use std::fs::File;
use std::io::{Read, stdin, Write};
use std::path::{Path, PathBuf};
use std::sync::Arc;
use futures_lite::AsyncReadExt;
use tokio_stream::{Stream, StreamExt};
use anyhow;
use std::time::Duration;
use iroh::base::node_addr::AddrInfoOptions;
use iroh::client::Doc;
use iroh::client::docs::{LiveEvent, ShareMode};
use iroh::docs::store::Query;
use iroh::node::Node;
use tokio::sync::Mutex;
use std::sync::mpsc::{channel, TryRecvError};
use iroh::blobs::store::fs::Store;
use iroh::docs::{Author, AuthorId, ContentStatus};
use iroh::net::key;
use iroh::node;
use notify::{Event, EventKind, RecursiveMode, Watcher, RecommendedWatcher};
use crate::sync_dir::SyncDir;
use tracing_subscriber::EnvFilter;

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    // Initialize logging
    tracing_subscriber::fmt()
        .with_env_filter(EnvFilter::from_default_env()
            .add_directive("info".parse().unwrap()))
        .init();

    let args: Vec<String> = env::args().skip(1).collect();
    
    // Check if user wants help
    if args.is_empty() || args[0] == "--help" || args[0] == "-h" {
        println!("Usage: {} <sync-directory> [ticket] [--web-port PORT] [--skip-diagnostics]", env::args().next().unwrap());
        println!("  sync-directory:     Directory to sync");
        println!("  ticket:            Optional ticket to connect to existing document");
        println!("  --web-port PORT:   Port for web interface (default: 3030)");
        println!("  --skip-diagnostics: Skip the initial diagnostic checks");
        println!("\nWeb interface will be available at:");
        println!("  Local:  http://localhost:PORT");
        println!("  Mobile: http://YOUR_PC_IP:PORT");
        println!("\nExample: {} ./my-sync-folder --web-port 8080", env::args().next().unwrap());
        return Ok(());
    }

    let base_path = env::current_dir().unwrap().join(args[0].parse::<String>().unwrap());
    println!("📁 BASE_PATH: {:?}", base_path);
    
    // Check if directory exists
    if !base_path.exists() {
        println!("❌ ERROR: Directory does not exist: {:?}", base_path);
        println!("Creating directory...");
        fs::create_dir_all(&base_path)?;
        println!("✅ Directory created!");
    }
    
    // Parse command line arguments
    let mut ticket_arg = None;
    let mut web_port = 3030u16;
    let mut skip_diagnostics = false;
    
    let mut i = 1;
    while i < args.len() {
        match args[i].as_str() {
            "--web-port" => {
                if i + 1 < args.len() {
                    web_port = args[i + 1].parse::<u16>().unwrap_or(3030);
                    i += 2;
                } else {
                    println!("⚠️  Warning: --web-port requires a port number, using default 3030");
                    i += 1;
                }
            },
            "--skip-diagnostics" => {
                skip_diagnostics = true;
                i += 1;
            },
            arg if !arg.starts_with("--") && ticket_arg.is_none() => {
                ticket_arg = Some(arg.to_string());
                i += 1;
            },
            _ => {
                i += 1;
            }
        }
    }
    
    println!("🔧 Creating SyncDir...");

    // Create SyncDir
    let sync_dir = if let Some(ticket) = ticket_arg {
        println!("🎫 Using provided ticket: {}", &ticket[..50.min(ticket.len())]); // Show first 50 chars
        SyncDir::new(base_path.clone(), Some(ticket)).await?
    } else {
        println!("📄 Creating new document (no ticket provided)");
        SyncDir::new(base_path.clone(), None).await?
    };
    
    println!("✅ SyncDir created successfully!");
    println!("🆔 Node ID: {}", sync_dir.node_id());
    println!("📄 Namespace: {}", sync_dir.namespace_id().await?);
    
    let sync_dir = Arc::new(Mutex::new(sync_dir));

    println!("📂 Loading local files...");
    let sync_dir_clone = Arc::clone(&sync_dir);
    {
        let sync_dir_lock = sync_dir_clone.lock().await;
        (*sync_dir_lock).load_local_files().await?;
    }
    println!("✅ Local files loaded!");

    // Run diagnostic checks unless skipped
    if !skip_diagnostics {
        println!("🔍 Running diagnostic checks...");
        let sync_dir_lock = sync_dir_clone.lock().await;

        // Check for conflicts
        (*sync_dir_lock).check_entry_conflicts().await?;

        // Comprehensive listing
        (*sync_dir_lock).list_all_entries_detailed().await?;

        // Test adding a specific missing file (if it exists)
        let test_file_path = base_path.join("emptymails/INBOX/new/email_369");
        if test_file_path.exists() {
            (*sync_dir_lock).test_add_missing_file("email_369").await?;
        }

        // Show debug contents
        println!("📋 Document contents:");
        (*sync_dir_lock).debug_contents().await?;

        println!("🔧 Adding all missing files...");
        (*sync_dir_lock).add_missing_files().await?;

        println!("📊 Comparing disk files with document entries...");
        (*sync_dir_lock).compare_disk_vs_document().await?;
        
        // Show final debug contents
        println!("📋 Final document contents:");
        (*sync_dir_lock).debug_contents().await?;
        
        drop(sync_dir_lock);
        println!("✅ Diagnostics complete!");
    } else {
        println!("⏩ Skipping diagnostics (--skip-diagnostics flag used)");
    }

    // Start web interface
    println!("🌐 Starting web interface on port {}...", web_port);
    let sync_dir_web = Arc::clone(&sync_dir);
    let web_handle = tokio::spawn(async move {
        if let Err(e) = crate::sync_dir::start_web_interface(sync_dir_web, web_port).await {
            eprintln!("❌ Web interface error: {}", e);
        }
    });

    // Subscribe to document events
    println!("📡 Subscribing to document events...");
    let mut stream = {
        let lock = sync_dir.lock().await;
        lock.subscribe().await?
    };
    println!("✅ Subscribed to document events!");

    // Get info for startup summary
    let (node_id_short, namespace_short, folder_name) = {
        let lock = sync_dir.lock().await;
        let node_id = lock.node_id().to_string();
        let namespace = lock.namespace_id().await?.to_string();
        let folder = base_path.file_name().unwrap_or_default().to_string_lossy().to_string();
        (node_id[..8].to_string(), namespace[..8].to_string(), folder)
    };

    // Print startup summary
    println!("\n╔══════════════════════════════════════╗");
    println!("║        🔄 IROH FILE SYNC ACTIVE       ║");
    println!("╠══════════════════════════════════════╣");
    println!("║ 🌐 Web Interface:                    ║");
    println!("║   Local:  http://localhost:{}      ║", web_port);
    println!("║   Mobile: http://YOUR_PC_IP:{}     ║", web_port);
    println!("║                                      ║");
    println!("║ 📁 Sync Folder: {:20} ║", folder_name);
    println!("║ 🆔 Node ID: {}...        ║", node_id_short);
    println!("║ 📄 Document: {}...       ║", namespace_short);
    println!("║                                      ║");
    println!("║ 💡 Tips:                            ║");
    println!("║   • Find your PC IP: ipconfig       ║");
    println!("║   • Type 'q' + Enter to quit        ║");
    println!("║   • Generate tickets in web UI      ║");
    println!("╚══════════════════════════════════════╝");
    println!();

    // Handle document events
    let sync_dir_clone_stream = Arc::clone(&sync_dir);
    let stream_task = tokio::spawn(async move {
        while let Some(item) = stream.next().await {
            match item {
                Ok(value) => {
                    println!("📡 Remote event: {:?}", value);
                    match value {
                        LiveEvent::InsertRemote { content_status, entry, .. } => {
                            if content_status == ContentStatus::Complete {
                                let sync_dir_lock = sync_dir_clone_stream.lock().await;
                                if let Err(e) = (*sync_dir_lock).write_remote_entry(entry.clone()).await {
                                    eprintln!("❌ Failed to write remote entry: {}", e);
                                }
                                drop(sync_dir_lock);
                            } else {
                                let sync_dir_lock = sync_dir_clone_stream.lock().await;
                                if let Err(e) = (*sync_dir_lock).handle_pending_entry(entry.clone()).await {
                                    eprintln!("❌ Failed to handle pending entry: {}", e);
                                }
                                drop(sync_dir_lock);
                            }
                        }
                        LiveEvent::ContentReady { .. } => {
                            let sync_dir_lock = sync_dir_clone_stream.lock().await;
                            if let Err(e) = (*sync_dir_lock).process_pending_entries().await {
                                eprintln!("❌ Failed to process pending entries: {}", e);
                            }
                            drop(sync_dir_lock);
                        }
                        LiveEvent::PendingContentReady => {
                            let sync_dir_lock = sync_dir_clone_stream.lock().await;
                            if let Err(e) = (*sync_dir_lock).process_pending_entries().await {
                                eprintln!("❌ Failed to process pending entries: {}", e);
                            }
                            drop(sync_dir_lock);
                        }
                        _ => {}
                    }
                }
                Err(e) => eprintln!("❌ Stream error: {}", e),
            }
        }
    });

    // File system watcher
    let sync_dir_clone_watcher = Arc::clone(&sync_dir);
    let watcher_base_path = base_path.clone();
    let watcher_task = tokio::spawn(async move {
        println!("👀 Starting file system watcher...");
        let (tx, rx) = channel();
        let mut watcher = RecommendedWatcher::new(
            move |res: Result<Event, notify::Error>| {
                if let Ok(event) = res {
                    let _ = tx.send(event);
                }
            },
            notify::Config::default()
        ).unwrap();
        
        if let Err(e) = watcher.watch(&watcher_base_path, RecursiveMode::Recursive) {
            eprintln!("❌ Failed to start file watcher: {}", e);
            return;
        }

        loop {
            match rx.try_recv() {
                Ok(event) => {
                    println!("📁 File event: {:?}", event);
                    let sync_dir_lock = sync_dir_clone_watcher.lock().await;
                    
                    for path in event.paths {
                        match event.kind {
                            EventKind::Create(_) => {
                                if let Err(e) = (*sync_dir_lock).handle_local_change(path.clone()).await {
                                    if !e.to_string().contains("newer entry exists") {
                                        eprintln!("❌ Failed to handle file creation {:?}: {}", path, e);
                                    }
                                }
                            }
                            EventKind::Modify(_) => {
                                if let Err(e) = (*sync_dir_lock).handle_local_change(path.clone()).await {
                                    if !e.to_string().contains("newer entry exists") {
                                        eprintln!("❌ Failed to handle file modification {:?}: {}", path, e);
                                    }
                                }
                            }
                            EventKind::Remove(_) => {
                                if let Err(e) = (*sync_dir_lock).handle_local_delete(path.clone()).await {
                                    eprintln!("❌ Failed to handle file deletion {:?}: {}", path, e);
                                }
                            }
                            _ => {}
                        }
                    }
                    drop(sync_dir_lock);
                }
                Err(e) => match e {
                    TryRecvError::Empty => {
                        tokio::time::sleep(Duration::from_millis(100)).await;
                    }
                    TryRecvError::Disconnected => {
                        println!("📁 File watcher disconnected");
                        break;
                    }
                },
            }
        }
    });

    // Enhanced input handling with commands
    println!("🎮 Available commands:");
    println!("  q     - Quit the application");
    println!("  s     - Show current status");
    println!("  d     - Show debug info");
    println!("  f     - Force refresh files");
    println!("  h     - Show help");
    println!();

    let mut input = String::new();
    loop {
        print!("iroh-sync> ");
        io::stdout().flush().unwrap();
        
        input.clear();
        match io::stdin().read_line(&mut input) {
            Ok(0) => break, // EOF
            Ok(_) => {
                match input.trim() {
                    "q" | "quit" | "exit" => {
                        println!("\n🛑 Shutting down...");
                        break;
                    }
                    "s" | "status" => {
                        let lock = sync_dir.lock().await;
                        match lock.get_sync_status().await {
                            Ok(status) => {
                                println!("📊 Status:");
                                println!("  🆔 Node ID: {}", status.node_id);
                                println!("  📄 Document: {}", status.namespace_id);
                                println!("  👥 Connected Peers: {}", status.connected_peers);
                                println!("  📁 Files: {}", status.total_files);
                                println!("  📂 Directories: {}", status.total_directories);
                            }
                            Err(e) => println!("❌ Failed to get status: {}", e),
                        }
                        drop(lock);
                    }
                    "d" | "debug" => {
                        let lock = sync_dir.lock().await;
                        if let Err(e) = lock.debug_contents().await {
                            println!("❌ Failed to show debug info: {}", e);
                        }
                        drop(lock);
                    }
                    "f" | "refresh" => {
                        println!("🔄 Force refreshing files...");
                        let lock = sync_dir.lock().await;
                        if let Err(e) = lock.load_local_files().await {
                            println!("❌ Failed to refresh files: {}", e);
                        } else {
                            println!("✅ Files refreshed!");
                        }
                        drop(lock);
                    }
                    "h" | "help" => {
                        println!("🎮 Available commands:");
                        println!("  q, quit, exit  - Quit the application");
                        println!("  s, status      - Show current sync status");
                        println!("  d, debug       - Show debug information");
                        println!("  f, refresh     - Force refresh local files");
                        println!("  h, help        - Show this help");
                    }
                    "" => {
                        // Empty line, just continue
                    }
                    cmd => {
                        println!("❓ Unknown command: '{}'. Type 'h' for help.", cmd);
                    }
                }
            }
            Err(e) => {
                eprintln!("❌ Failed to read input: {}", e);
                break;
            }
        }
    }
    
    // Cleanup
    println!("🧹 Cleaning up...");
    
    // Cancel tasks
    stream_task.abort();
    watcher_task.abort();
    web_handle.abort();
    
    // Shutdown sync system
    let sync_dir_clone = Arc::clone(&sync_dir);
    let sync_dir_lock = sync_dir_clone.lock().await;
    if let Err(e) = (*sync_dir_lock).shutdown().await {
        eprintln!("❌ Shutdown error: {}", e);
    }
    drop(sync_dir_lock);
    
    println!("✅ Shutdown complete! Goodbye!");
    Ok(())
}