use std::collections::{HashMap, HashSet};
use std::fs;
use std::path::{Path, PathBuf};
use std::sync::Arc;
use std::time::{Duration, Instant};

use anyhow::{Context, Result};
use bytes::Bytes;
use futures_lite::{Stream, StreamExt};
use tokio::sync::Mutex;
use tokio::io::AsyncReadExt;
use tracing::{debug, error, info, warn};

use warp::Filter;
use serde::{Deserialize, Serialize};
use futures_util::TryStreamExt;
use tokio_util::io::StreamReader;

// Updated imports for iroh 0.28
use iroh::{
    base::{
        node_addr::AddrInfoOptions,
        ticket::Ticket,
    },
    client::{
        blobs::{WrapOption, AddProgress},
        docs::{Doc, Entry, LiveEvent, ShareMode},
        Iroh,
    },
    docs::{
        Author, AuthorId, ContentStatus, DocTicket, NamespaceId,
        store::Query,
    },
    net::{NodeId, NodeAddr},
    node::{Node, Builder as NodeBuilder},
    blobs::Hash,
};

/// File metadata stored in docs
#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
pub struct FileMetadata {
    pub size: u64,
    pub modified: u64,
}

#[derive(Serialize)]
pub struct FileInfo {
    pub name: String,
    pub size: u64,
    pub is_directory: bool,
    pub path: String,
}

#[derive(Serialize)]
pub struct SyncStatus {
    pub node_id: String,
    pub namespace_id: String,
    pub connected_peers: usize,
    pub total_files: usize,
    pub total_directories: usize,
}

#[derive(Deserialize)]
pub struct ConnectRequest {
    pub ticket: String,
}

/// Main sync directory handler
pub struct SyncDir {
    base_path: PathBuf,
    node: Node<iroh::blobs::store::fs::Store>,
    client: Iroh,
    author: AuthorId,
    doc: Arc<Mutex<Doc>>,
    // Track pending entries waiting for content
    pending_entries: Arc<Mutex<HashMap<String, Entry>>>,
    // Track recently written files to ignore file watcher events
    recently_written: Arc<Mutex<HashMap<PathBuf, Instant>>>,
    // Track connected peers
    connected_peers: Arc<Mutex<HashSet<NodeId>>>,
    failed_files: Arc<Mutex<Vec<(String, String)>>>,
}

impl SyncDir {
    /// Create a new SyncDir instance
    pub async fn new(base_path: PathBuf, ticket: Option<String>) -> Result<Self> {
        info!("Initializing SyncDir at {:?}", base_path);

        // Create data directory in current working directory, not in sync directory
        let data_path = std::env::current_dir()?.join(".iroh-sync");
        fs::create_dir_all(&data_path)?;
        info!("Using data directory: {:?}", data_path);

        // Create the node
        let node = Node::persistent(&data_path)
            .await?
            .enable_docs()
            .spawn()
            .await?;

        let cloned_node = node.clone();
        let client = cloned_node.client();
        
        // Get or create author
        let author = client.authors().default().await?;
        info!("Using author: {}", author);

        // Get or create document
        let doc = match ticket {
            Some(ticket_str) => {
                info!("Importing from ticket: {}", ticket_str);
                let ticket: DocTicket = ticket_str.parse()
                    .context("Failed to parse document ticket")?;
                
                client.docs().import(ticket).await?
            }
            None => {
                info!("Creating new document");
                let doc = client.docs().create().await?;
                
                // Generate sharing ticket
                let ticket = doc.share(ShareMode::Write, AddrInfoOptions::RelayAndAddresses).await?;
                println!("=== DOCUMENT TICKET ===");
                println!("{}", ticket);
                println!("Save this ticket to share with other peers!");
                println!("=======================");
                info!("Created new doc with ticket: {}", ticket);
                
                doc
            }
        };

        let doc = Arc::new(Mutex::new(doc));

        Ok(SyncDir {
            base_path,
            node,
            client: client.clone(),
            author,
            doc,
            pending_entries: Arc::new(Mutex::new(HashMap::new())),
            recently_written: Arc::new(Mutex::new(HashMap::new())),
            connected_peers: Arc::new(Mutex::new(HashSet::new())),
            failed_files: Arc::new(Mutex::new(Vec::new())),
        })
    }

    /// Load all local files into the sync system
    pub async fn load_local_files(&self) -> Result<()> {
        info!("Loading local files from {:?}", self.base_path);
        self.scan_and_add_directory(&self.base_path).await?;
        info!("Local file loading complete");
        Ok(())
    }

        /// Recursively scan and add files from a directory
    async fn scan_and_add_directory(&self, dir_path: &Path) -> Result<()> {
    info!("Scanning directory: {:?}", dir_path);
    let entries = fs::read_dir(dir_path)?;
    let mut file_count = 0;
    let mut dir_count = 0;
    let mut skipped_count = 0;

    for entry in entries {
        let entry = entry?;
        let path = entry.path();
        
        // Add detailed logging for each file
        info!("Processing: {:?}", path);
        
        let metadata = match entry.metadata() {
            Ok(m) => m,
            Err(e) => {
                error!("Failed to get metadata for {:?}: {}", path, e);
                skipped_count += 1;
                continue;
            }
        };

        if metadata.is_dir() {
            dir_count += 1;
            if let Err(e) = self.add_directory_to_sync(&path).await {
                if !e.to_string().contains("newer entry exists") {
                    error!("Failed to add directory {:?}: {}", path, e);
                    skipped_count += 1;
                }
            }
            Box::pin(self.scan_and_add_directory(&path)).await?;
        } else if metadata.is_file() {
            file_count += 1;
            // Log file details
            info!("Adding file: {:?} (size: {} bytes)", path, metadata.len());
            
            if let Err(e) = self.add_file_to_sync(&path).await {
                if !e.to_string().contains("newer entry exists") {
                    error!("Failed to add file {:?}: {}", path, e);
                    skipped_count += 1;
                }
            }
        } else {
            info!("Skipping special file type: {:?}", path);
            skipped_count += 1;
        }
    }
    
    info!("Directory scan complete for {:?}: {} files, {} dirs, {} skipped", 
          dir_path, file_count, dir_count, skipped_count);
    Ok(())
}

    /// Add a directory to the sync system
    async fn add_directory_to_sync(&self, dir_path: &Path) -> Result<()> {
        let relative_path = self.get_relative_path(dir_path)?;
        let key = format!("d{}", relative_path.to_string_lossy()); // Use "d" prefix for directories

        // Check if this directory entry already exists
        if self.entry_exists(&key).await? {
            debug!("Directory entry already exists: {}", relative_path.display());
            return Ok(());
        }

        // Store directory marker with minimal content
        let doc = self.doc.lock().await;
        match doc.set_bytes(self.author, key.clone(), vec![0u8]).await {
            Ok(_hash) => {
                info!("Added directory: {}", relative_path.display());
            }
            Err(e) if e.to_string().contains("newer entry exists") => {
                debug!("Directory entry already exists (detected during write): {}", relative_path.display());
            }
            Err(e) => {
                error!("Failed to add directory {:?}: {}", dir_path, e);
                return Err(e.into());
            }
        }
        drop(doc);

        Ok(())
    }

    /// Add a single file to the sync system
    async fn add_file_to_sync(&self, file_path: &Path) -> Result<()> {
    let relative_path = self.get_relative_path(file_path)?;
    let key = relative_path.to_string_lossy().to_string();

    info!("Attempting to add file: {} (full path: {:?})", key, file_path);

    // Skip Windows Zone.Identifier files
    if key.ends_with(":Zone.Identifier") {
        debug!("Skipping Zone.Identifier file: {}", key);
        return Ok(());
    }

    // Check for other problematic patterns
    if key.contains('\0') {
        warn!("Skipping file with null character: {:?}", file_path);
        return Ok(());
    }

    // Read file content with better error handling
    let content = match fs::read(file_path) {
        Ok(c) => c,
        Err(e) => {
            error!("Failed to read file {:?}: {}", file_path, e);
            return Err(e.into());
        }
    };
    
    info!("Read file successfully: {} ({} bytes)", key, content.len());

    // Handle empty files - Iroh doesn't allow completely empty entries
    let content_to_store = if content.is_empty() {
        vec![0u8] // Single null byte as placeholder
    } else {
        content
    };

    // Check if this entry already exists with the same content
    if let Ok(true) = self.entry_exists_with_content(&key, &content_to_store).await {
        debug!("File entry already exists with same content: {}", key);
        return Ok(());
    }

    // Store content directly in doc (for iroh 0.28)
    let doc = self.doc.lock().await;
    match doc.set_bytes(self.author, key.clone(), content_to_store).await {
        Ok(_hash) => {
            info!("Added file: {}", key);
        }
        Err(e) if e.to_string().contains("newer entry exists") => {
            debug!("File entry already exists (detected during write): {}", key);
        }
        Err(e) => {
            error!("Failed to add file {:?}: {}", file_path, e);
            return Err(e.into());
        }
    }
    drop(doc);

    Ok(())  // This was missing!
}


// Add this method to report failed files
pub async fn report_failed_files(&self) -> Result<()> {
    let failed = self.failed_files.lock().await;
    if failed.is_empty() {
        println!("No failed files recorded during loading.");
    } else {
        println!("\n=== FAILED FILES REPORT ===");
        println!("Total failed files: {}", failed.len());
        for (file, reason) in failed.iter() {
            println!("  - {}: {}", file, reason);
        }
        println!("=== END REPORT ===\n");
    }
    Ok(())
}
    /// Download a blob from connected peers - Fixed for Iroh 0.28
    async fn download_blob(&self, hash: Hash) -> Result<Vec<u8>> {
        info!("Attempting to download blob: {}", hash);
        
        // Get list of connected peers
        let peers = self.connected_peers.lock().await.clone();
        
        if peers.is_empty() {
            return Err(anyhow::anyhow!("No connected peers to download from"));
        }
        
        // Try to download from each connected peer
        for peer_id in peers {
            info!("Trying to download from peer: {}", peer_id);
            
            // Create a NodeAddr with just the peer ID
            // The node should already know how to reach this peer since they're connected
            let node_addr = iroh::net::NodeAddr::new(peer_id);
            
            match self.client.blobs().download(hash, node_addr).await {
                Ok(mut progress) => {
                    // Wait for download to complete by consuming the stream
                    while let Some(event) = progress.next().await {
                        match event {
                            Ok(progress_event) => {
                                debug!("Download progress: {:?}", progress_event);
                            }
                            Err(e) => {
                                warn!("Download progress error: {}", e);
                            }
                        }
                    }
                    
                    // After the progress stream completes, the blob should be available
                    match self.client.blobs().read(hash).await {
                        Ok(mut reader) => {
                            let mut content = Vec::new();
                            reader.read_to_end(&mut content).await?;
                            info!("Successfully downloaded blob: {} ({} bytes) from peer {}", hash, content.len(), peer_id);
                            return Ok(content);
                        }
                        Err(e) => {
                            warn!("Failed to read blob after download from peer {}: {}", peer_id, e);
                            continue; // Try next peer
                        }
                    }
                }
                Err(e) => {
                    warn!("Failed to start blob download from peer {}: {}", peer_id, e);
                    continue; // Try next peer
                }
            }
        }
        
        Err(anyhow::anyhow!("Failed to download blob from any connected peer"))
    }

    /// Write a received entry to the local filesystem - with directory support
    pub async fn write_remote_entry(&self, entry: Entry) -> Result<()> {
        let key = String::from_utf8_lossy(entry.key()).to_string();
        info!("Processing remote entry: {}", key);
        
        // Check if this is a directory entry (starts with "d")
        if key.starts_with("d") {
            // Extract the directory path (remove "d" prefix)
            let dir_path = &key[1..];
            let full_dir_path = self.base_path.join(dir_path);
            
            // Create the directory
            fs::create_dir_all(&full_dir_path)?;
            
            // Track that we just created this directory
            let mut recently_written = self.recently_written.lock().await;
            recently_written.insert(full_dir_path.clone(), Instant::now());
            drop(recently_written);
            
            info!("Created directory: {}", full_dir_path.display());
            return Ok(());
        }
        
        // Handle file entries
        let content: Vec<u8> = {
            let doc = self.doc.lock().await;
            match entry.content_bytes(&*doc).await {
                Ok(bytes_content) => {
                    bytes_content.to_vec()
                }
                Err(e) => {
                    error!("Failed to get content for {}: {}", key, e);
                    drop(doc);
                    
                    let hash = entry.content_hash();
                    info!("Content not available locally, downloading blob: {}", hash);
                    
                    match self.download_blob(hash).await {
                        Ok(downloaded_content) => downloaded_content,
                        Err(download_err) => {
                            error!("Failed to download blob: {}", download_err);
                            return Err(download_err);
                        }
                    }
                }
            }
        };
        
        // Handle empty file placeholder
        let actual_content = if content == vec![0u8] {
            // This was an empty file stored with placeholder
            Vec::new()
        } else {
            content
        };
        
        // Construct the full file path
        let file_path = self.base_path.join(&key);
        
        // Create parent directories if needed
        if let Some(parent) = file_path.parent() {
            fs::create_dir_all(parent)?;
        }
        
        // Write the file
        fs::write(&file_path, &actual_content)?;
        
        // Track that we just wrote this file
        let mut recently_written = self.recently_written.lock().await;
        recently_written.insert(file_path.clone(), Instant::now());
        drop(recently_written);
        
        info!("Wrote file: {} ({} bytes)", file_path.display(), actual_content.len());
        Ok(())
    }

    /// Handle pending entries - store them for later processing
    pub async fn handle_pending_entry(&self, entry: Entry) -> Result<()> {
        let key = String::from_utf8_lossy(entry.key()).to_string();
        let mut pending = self.pending_entries.lock().await;
        pending.insert(key.clone(), entry);
        info!("Stored pending entry: {}", key);
        Ok(())
    }

    /// Process all pending entries when content becomes ready
    pub async fn process_pending_entries(&self) -> Result<()> {
        let mut pending = self.pending_entries.lock().await;
        let entries_to_process: Vec<_> = pending.drain().collect();
        drop(pending);
        
        info!("Processing {} pending entries", entries_to_process.len());
        
        let mut failed_entries = Vec::new();
        
        for (key, entry) in entries_to_process {
            match self.write_remote_entry(entry.clone()).await {
                Ok(()) => {
                    info!("Successfully processed pending entry: {}", key);
                }
                Err(e) => {
                    error!("Failed to process pending entry {}: {}", key, e);
                    // Re-add to pending if it's because of no connected peers
                    if e.to_string().contains("No connected peers") {
                        failed_entries.push((key, entry));
                    }
                }
            }
        }
        
        // Re-add failed entries back to pending
        if !failed_entries.is_empty() {
            let mut pending = self.pending_entries.lock().await;
            for (key, entry) in failed_entries {
                pending.insert(key, entry);
            }
            info!("Re-added {} entries to pending due to no connected peers", pending.len());
        }
        
        Ok(())
    }

    /// Track a connected peer
    pub async fn add_peer(&self, peer_id: NodeId) {
        let mut peers = self.connected_peers.lock().await;
        peers.insert(peer_id);
        info!("Added connected peer: {}", peer_id);
    }

    /// Remove a disconnected peer
    pub async fn remove_peer(&self, peer_id: NodeId) {
        let mut peers = self.connected_peers.lock().await;
        peers.remove(&peer_id);
        info!("Removed disconnected peer: {}", peer_id);
    }

    /// Handle local file and directory changes
    pub async fn handle_local_change(&self, path: PathBuf) -> Result<()> {
        // Check if this file was recently written by us (ignore for 2 seconds)
        if self.was_recently_written(&path).await {
            debug!("Ignoring file watcher event for recently written file: {:?}", path);
            return Ok(());
        }
        
        if path.is_file() {
            if let Err(e) = self.add_file_to_sync(&path).await {
                // Only log as error if it's not a "newer entry exists" issue
                if !e.to_string().contains("newer entry exists") {
                    error!("Failed to handle file change {:?}: {}", path, e);
                }
            }
        } else if path.is_dir() {
            if let Err(e) = self.add_directory_to_sync(&path).await {
                // Only log as error if it's not a "newer entry exists" issue  
                if !e.to_string().contains("newer entry exists") {
                    error!("Failed to handle directory change {:?}: {}", path, e);
                }
            }
        }
        Ok(())
    }

    /// Check if a file was recently written by the sync system
    async fn was_recently_written(&self, path: &Path) -> bool {
        let mut recently_written = self.recently_written.lock().await;
        
        // Clean up old entries (older than 2 seconds)
        let now = Instant::now();
        recently_written.retain(|_, time| now.duration_since(*time) < Duration::from_secs(2));
        
        // Check if this path was recently written
        recently_written.contains_key(path)
    }

    /// Handle local file and directory deletion
    pub async fn handle_local_delete(&self, path: PathBuf) -> Result<()> {
        let relative_path = self.get_relative_path(&path)?;
        
        // Try both file and directory keys (since we don't know which it was)
        let file_key = relative_path.to_string_lossy().to_string();
        let dir_key = format!("d{}", relative_path.to_string_lossy());

        info!("Removing: {}", relative_path.display());

        let doc = self.doc.lock().await;
        
        // Try to delete as file first
        if let Err(_) = doc.del(self.author, file_key.clone()).await {
            // If file deletion fails, try as directory
            let _ = doc.del(self.author, dir_key.clone()).await;
        }
        
        drop(doc);

        Ok(())
    }

    /// Handle file rename
    pub async fn handle_rename(&self, src: PathBuf, dst: PathBuf) -> Result<()> {
        info!("Renaming: {:?} -> {:?}", src, dst);
        
        // Delete old entry
        self.handle_local_delete(src).await?;
        
        // Add new entry
        if dst.is_file() {
            self.handle_local_change(dst).await?;
        } else if dst.is_dir() {
            self.handle_local_change(dst).await?;
        }
        
        Ok(())
    }

    /// Check if an entry exists in the document
    async fn entry_exists(&self, key: &str) -> Result<bool> {
        let doc = self.doc.lock().await;
        let mut entries = doc.get_many(Query::single_latest_per_key().key_exact(key.to_string())).await?;
        let exists = entries.next().await.is_some();
        drop(doc);
        Ok(exists)
    }

    /// Check if an entry exists with the same content
    async fn entry_exists_with_content(&self, key: &str, content: &[u8]) -> Result<bool> {
        let doc = self.doc.lock().await;
        let mut entries = doc.get_many(Query::single_latest_per_key().key_exact(key.to_string())).await?;
        
        if let Some(entry_result) = entries.next().await {
            match entry_result {
                Ok(entry) => {
                    match entry.content_bytes(&*doc).await {
                        Ok(existing_content) => {
                            let exists_with_same_content = existing_content.as_ref() == content;
                            drop(doc);
                            return Ok(exists_with_same_content);
                        }
                        Err(_) => {
                            drop(doc);
                            return Ok(false);
                        }
                    }
                }
                Err(_) => {
                    drop(doc);
                    return Ok(false);
                }
            }
        }
        
        drop(doc);
        Ok(false)
    }

    /// Get relative path from base
    fn get_relative_path(&self, path: &Path) -> Result<PathBuf> {
        path.strip_prefix(&self.base_path)
            .map(|p| p.to_path_buf())
            .context("Failed to get relative path")
    }

    /// Subscribe to document events
    pub async fn subscribe(&self) -> Result<impl Stream<Item = Result<LiveEvent>>> {
        let doc = self.doc.lock().await;
        let stream = doc.subscribe().await?;
        drop(doc);
        Ok(stream)
    }

    /// Get the node ID
    pub fn node_id(&self) -> NodeId {
        self.node.node_id()
    }

    /// Get the document namespace ID
    pub async fn namespace_id(&self) -> Result<NamespaceId> {
        let doc = self.doc.lock().await;
        Ok(doc.id())
    }

    /// Shutdown the sync system
    pub async fn shutdown(&self) -> Result<()> {
        info!("Shutting down sync system...");
        
        // Shutdown node (clone it since shutdown takes ownership)
        self.node.clone().shutdown().await?;
        
        info!("Shutdown complete");
        Ok(())
    }

    /// Debug: show document contents
    pub async fn debug_contents(&self) -> Result<()> {
    let doc = self.doc.lock().await;

    // IMPORTANT: Use Query::all() instead of Query::single_latest_per_key()
    // to see ALL entries, not just the latest per key
    let mut entries = doc.get_many(Query::all()).await?;
    let mut file_count = 0;
    let mut dir_count = 0;
    let mut total_count = 0;

    println!("\n=== DOCUMENT CONTENTS ===");
    while let Some(entry) = entries.next().await {
        match entry {
            Ok(e) => {
                total_count += 1;
                let key = String::from_utf8_lossy(e.key());
                let entry_type = if key.starts_with("d") { 
                    dir_count += 1;
                    "DIR " 
                } else { 
                    file_count += 1;
                    "FILE" 
                };
                
                // For now, let's see ALL entries to debug
                println!("  [{}] {} {} (author: {})", 
                    total_count, 
                    entry_type, 
                    key,
                    &e.author().to_string()[..8] // Show first 8 chars of author ID
                );
            }
            Err(e) => println!("  Error reading entry: {}", e),
        }
    }
    
    println!("\nFiles: {}, Directories: {}, Total: {}", file_count, dir_count, total_count);
    println!("========================\n");

    drop(doc);
    Ok(())
}
/// Add this temporary debug function to SyncDir
pub async fn debug_missing_files(&self) -> Result<()> {
    println!("\n=== INVESTIGATING MISSING FILES ===");
    
    let doc = self.doc.lock().await;
    
    // Try different query methods
    println!("\n1. Using Query::all():");
    let mut all_entries = doc.get_many(Query::all()).await?;
    let mut count_all = 0;
    while let Some(entry) = all_entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key());
            if key.contains("email_369") {
                println!("   FOUND via Query::all(): {}", key);
            }
        }
        count_all += 1;
    }
    println!("   Total entries with Query::all(): {}", count_all);
    
    // Try with single_latest_per_key
    println!("\n2. Using Query::single_latest_per_key():");
    let mut latest_entries = doc.get_many(Query::single_latest_per_key()).await?;
    let mut count_latest = 0;
    let mut found_369 = false;
    while let Some(entry) = latest_entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key());
            if key.contains("email_369") {
                println!("   FOUND via Query::single_latest_per_key(): {}", key);
                println!("   Author: {}", e.author());
                println!("   Timestamp: {:?}", e.timestamp());
                found_369 = true;
            }
        }
        count_latest += 1;
    }
    println!("   Total entries with Query::single_latest_per_key(): {}", count_latest);
    
    if !found_369 {
        println!("\n3. Trying direct key query for email_369:");
        let key = "emptymails/INBOX/new/email_369";
        let mut direct_query = doc.get_many(Query::single_latest_per_key().key_exact(key)).await?;
        if let Some(entry_result) = direct_query.next().await {
            match entry_result {
                Ok(e) => {
                    println!("   FOUND via direct key query!");
                    println!("   Key: {}", String::from_utf8_lossy(e.key()));
                    println!("   Author: {}", e.author());
                    println!("   Hash: {}", e.content_hash());
                    println!("   Timestamp: {:?}", e.timestamp());
                }
                Err(e) => {
                    println!("   ERROR in direct query: {}", e);
                }
            }
        } else {
            println!("   NOT FOUND via direct key query");
        }
    }
    
    // Check for files with similar patterns
    println!("\n4. Checking pattern of missing files in INBOX/new:");
    let mut email_files: Vec<(u32, String)> = Vec::new();
    let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
    
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key());
            if key.contains("emptymails/INBOX/new/email_") && !key.contains("Zone.Identifier") {
                // Extract the number
                if let Some(num_str) = key.split("email_").nth(1) {
                    if let Ok(num) = num_str.parse::<u32>() {
                        email_files.push((num, key.to_string()));
                    }
                }
            }
        }
    }
    
    email_files.sort_by_key(|k| k.0);
    
    println!("\n   Email files found (sorted by number):");
    let mut last_num = 0;
    for (num, key) in &email_files {
        if last_num > 0 && *num > last_num + 1 {
            println!("   --- GAP: missing {} to {} ---", last_num + 1, num - 1);
        }
        println!("   {}: {}", num, key);
        last_num = *num;
    }
    
    drop(doc);
    println!("\n=== END INVESTIGATION ===\n");
    Ok(())
}

pub async fn compare_disk_vs_document(&self) -> Result<()> {
    println!("\n=== DISK vs DOCUMENT COMPARISON ===");
    
    // First, count files on disk
    let mut disk_files = HashSet::new();
    let mut disk_dirs = HashSet::new();
    let mut zone_identifier_count = 0;
    
    self.scan_disk_recursive(&self.base_path, &mut disk_files, &mut disk_dirs, &mut zone_identifier_count)?;
    
    println!("\nDISK STATISTICS:");
    println!("  Total files on disk: {}", disk_files.len() + zone_identifier_count);
    println!("  Zone.Identifier files: {}", zone_identifier_count);
    println!("  Actual content files: {}", disk_files.len());
    println!("  Directories: {}", disk_dirs.len());
    
    // Now count entries in the document
    let doc = self.doc.lock().await;
    let mut doc_files = HashSet::new();
    let mut doc_dirs = HashSet::new();
    
    let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key()).to_string();
            if key.starts_with("d") {
                doc_dirs.insert(key[1..].to_string());
            } else {
                doc_files.insert(key);
            }
        }
    }
    drop(doc);
    
    println!("\nDOCUMENT STATISTICS:");
    println!("  Files in document: {}", doc_files.len());
    println!("  Directories in document: {}", doc_dirs.len());
    println!("  Total entries: {}", doc_files.len() + doc_dirs.len());
    
    // Find differences
    println!("\nDIFFERENCES:");
    println!("  Files on disk but not in doc: {}", disk_files.difference(&doc_files).count());
    println!("  Files in doc but not on disk: {}", doc_files.difference(&disk_files).count());
    
    // Show missing files
    let missing_from_doc: Vec<_> = disk_files.difference(&doc_files).collect();
    if !missing_from_doc.is_empty() {
        println!("\nFILES ON DISK BUT MISSING FROM DOCUMENT:");
        // Group by directory for easier reading
        let mut by_dir: std::collections::HashMap<String, Vec<String>> = std::collections::HashMap::new();
        for file in missing_from_doc {
            let dir = file.rsplitn(2, '/').nth(1).unwrap_or("root").to_string();
            by_dir.entry(dir).or_insert_with(Vec::new).push(file.clone());
        }
        
        for (dir, mut files) in by_dir {
            files.sort();
            println!("\n  In {}:", dir);
            for file in files {
                println!("    - {}", file);
            }
        }
    }
    
    // Check for the specific problem area
    println!("\n=== INBOX/new SPECIFIC ANALYSIS ===");
    let inbox_prefix = "emptymails/INBOX/new/";
    let disk_inbox: Vec<_> = disk_files.iter()
        .filter(|f| f.starts_with(inbox_prefix))
        .collect();
    let doc_inbox: Vec<_> = doc_files.iter()
        .filter(|f| f.starts_with(inbox_prefix))
        .collect();
    
    println!("  Files in INBOX/new on disk: {}", disk_inbox.len());
    println!("  Files in INBOX/new in document: {}", doc_inbox.len());
    
    // Extract email numbers and find gaps
    let mut disk_email_nums: Vec<u32> = Vec::new();
    let mut doc_email_nums: Vec<u32> = Vec::new();
    
    for file in disk_inbox {
        if let Some(num_str) = file.strip_prefix(&format!("{}email_", inbox_prefix)) {
            if let Ok(num) = num_str.parse::<u32>() {
                disk_email_nums.push(num);
            }
        }
    }
    
    for file in doc_inbox {
        if let Some(num_str) = file.strip_prefix(&format!("{}email_", inbox_prefix)) {
            if let Ok(num) = num_str.parse::<u32>() {
                doc_email_nums.push(num);
            }
        }
    }
    
    disk_email_nums.sort();
    doc_email_nums.sort();
    
    println!("\n  Email number ranges:");
    if !disk_email_nums.is_empty() {
        println!("    On disk: {} to {}", disk_email_nums.first().unwrap(), disk_email_nums.last().unwrap());
    }
    if !doc_email_nums.is_empty() {
        println!("    In document: {} to {}", doc_email_nums.first().unwrap(), doc_email_nums.last().unwrap());
    }
    
    println!("\n=== END COMPARISON ===\n");
    Ok(())
}

/// Helper function to recursively scan disk
fn scan_disk_recursive(
    &self,
    dir_path: &Path,
    files: &mut HashSet<String>,
    dirs: &mut HashSet<String>,
    zone_identifier_count: &mut usize,
) -> Result<()> {
    let entries = fs::read_dir(dir_path)?;
    
    for entry in entries {
        let entry = entry?;
        let path = entry.path();
        let metadata = entry.metadata()?;
        
        if metadata.is_dir() {
            let relative = self.get_relative_path(&path)?;
            dirs.insert(relative.to_string_lossy().to_string());
            self.scan_disk_recursive(&path, files, dirs, zone_identifier_count)?;
        } else if metadata.is_file() {
            let relative = self.get_relative_path(&path)?;
            let relative_str = relative.to_string_lossy().to_string();
            
            if relative_str.ends_with(":Zone.Identifier") {
                *zone_identifier_count += 1;
            } else {
                files.insert(relative_str);
            }
        }
    }
    
    Ok(())
}

/// Analyze missing files in detail
pub async fn analyze_missing_files(&self) -> Result<()> {
    println!("\n=== DETAILED MISSING FILE ANALYSIS ===");
    
    // Get all files from disk
    let mut disk_files = HashSet::new();
    let mut disk_dirs = HashSet::new();
    let mut zone_count = 0;
    self.scan_disk_recursive(&self.base_path, &mut disk_files, &mut disk_dirs, &mut zone_count)?;
    
    // Get all files from document
    let doc = self.doc.lock().await;
    let mut doc_files = HashSet::new();
    let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key()).to_string();
            if !key.starts_with("d") {
                doc_files.insert(key);
            }
        }
    }
    drop(doc);
    
    // Focus on INBOX/new
    let inbox_prefix = "emptymails/INBOX/new/";
    let missing_inbox: Vec<String> = disk_files.difference(&doc_files)
        .filter(|f| f.starts_with(inbox_prefix))
        .cloned()
        .collect();
    
    println!("Total missing files in INBOX/new: {}", missing_inbox.len());
    
    // Analyze patterns
    let mut missing_by_size: std::collections::HashMap<String, Vec<String>> = std::collections::HashMap::new();
    let mut size_errors = 0;
    let mut total_size_missing = 0u64;
    
    for file in &missing_inbox {
        let full_path = self.base_path.join(file);
        match fs::metadata(&full_path) {
            Ok(meta) => {
                let size = meta.len();
                total_size_missing += size;
                let size_category = match size {
                    0 => "empty (0 bytes)".to_string(),
                    1..=1024 => "tiny (1B-1KB)".to_string(),
                    1025..=102400 => "small (1KB-100KB)".to_string(),
                    102401..=1048576 => "medium (100KB-1MB)".to_string(),
                    _ => format!("large (>1MB, exactly {} bytes)", size),
                };
                missing_by_size.entry(size_category).or_insert_with(Vec::new).push(file.clone());
            }
            Err(e) => {
                println!("ERROR reading metadata for {}: {}", file, e);
                size_errors += 1;
            }
        }
    }
    
    println!("\nMissing files by size category:");
    for (category, files) in &missing_by_size {
        println!("  {}: {} files", category, files.len());
        // Show a few examples
        for file in files.iter().take(3) {
            println!("    - {}", file);
        }
        if files.len() > 3 {
            println!("    ... and {} more", files.len() - 3);
        }
    }
    
    println!("\nTotal size of missing files: {} bytes ({:.2} MB)", 
             total_size_missing, total_size_missing as f64 / 1_048_576.0);
    
    // Check if these files have anything special in common
    println!("\nChecking file patterns:");
    
    // Extract email numbers from missing files
    let mut missing_numbers: Vec<u32> = Vec::new();
    for file in &missing_inbox {
        if let Some(num_str) = file.strip_prefix(&format!("{}email_", inbox_prefix)) {
            if let Ok(num) = num_str.parse::<u32>() {
                missing_numbers.push(num);
            }
        }
    }
    missing_numbers.sort();
    
    // Find continuous ranges
    if !missing_numbers.is_empty() {
        println!("\nMissing email numbers (showing ranges):");
        let mut ranges = Vec::new();
        let mut start = missing_numbers[0];
        let mut end = missing_numbers[0];
        
        for &num in &missing_numbers[1..] {
            if num == end + 1 {
                end = num;
            } else {
                ranges.push((start, end));
                start = num;
                end = num;
            }
        }
        ranges.push((start, end));
        
        for (start, end) in ranges {
            if start == end {
                println!("  - {}", start);
            } else {
                println!("  - {} to {} ({} files)", start, end, end - start + 1);
            }
        }
    }
    
    // Try to read a few missing files to check content
    println!("\nSampling missing files to check readability:");
    for file in missing_inbox.iter().take(5) {
        let full_path = self.base_path.join(file);
        print!("  {}: ", file);
        match fs::read(&full_path) {
            Ok(content) => {
                println!("Readable, {} bytes, starts with: {:?}", 
                         content.len(), 
                         String::from_utf8_lossy(&content[..content.len().min(50)]));
            }
            Err(e) => {
                println!("ERROR reading: {}", e);
            }
        }
    }
    
    println!("\n=== END ANALYSIS ===\n");
    Ok(())
}

/// Check if there are multiple versions or conflicts for entries
pub async fn check_entry_conflicts(&self) -> Result<()> {
    println!("\n=== CHECKING FOR ENTRY CONFLICTS ===");
    
    let doc = self.doc.lock().await;
    
    // First, let's check a specific missing file
    let test_key = "emptymails/INBOX/new/email_369";
    println!("\n1. Checking all versions of {}", test_key);
    
    // Use Query::all() to get ALL entries, not just latest
    let mut all_entries = doc.get_many(Query::all().key_exact(test_key)).await?;
    let mut version_count = 0;
    
    while let Some(entry) = all_entries.next().await {
        if let Ok(e) = entry {
            version_count += 1;
            println!("   Version {}: author={}, timestamp={:?}, hash={}", 
                     version_count,
                     &e.author().to_string()[..8],
                     e.timestamp(),
                     e.content_hash());
        }
    }
    
    if version_count == 0 {
        println!("   NO VERSIONS FOUND for {}", test_key);
    }
    
    // Now check if we have multiple authors
    println!("\n2. Checking authors in the document");
    let mut authors = HashSet::new();
    let mut entries_by_author: HashMap<String, usize> = HashMap::new();
    
    let mut all_entries = doc.get_many(Query::all()).await?;
    while let Some(entry) = all_entries.next().await {
        if let Ok(e) = entry {
            let author_str = e.author().to_string();
            authors.insert(e.author());
            *entries_by_author.entry(author_str).or_insert(0) += 1;
        }
    }
    
    println!("   Found {} different authors:", authors.len());
    for (author, count) in &entries_by_author {
        println!("   - {}: {} entries", &author[..8], count);
    }
    
    // Check if our current author matches
    println!("\n3. Current author: {}", &self.author.to_string()[..8]);
    
    // Let's check the difference between Query::all() and Query::single_latest_per_key()
    println!("\n4. Comparing query methods for INBOX/new files");
    
    let mut all_inbox_entries = HashMap::new();
    let mut latest_inbox_entries = HashMap::new();
    
    // Get ALL entries
    let mut entries = doc.get_many(Query::all()).await?;
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key()).to_string();
            if key.starts_with("emptymails/INBOX/new/email_") && !key.contains("Zone") {
                all_inbox_entries.entry(key.clone())
                    .or_insert_with(Vec::new)
                    .push(e.author());
            }
        }
    }
    
    // Get LATEST entries
    let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key()).to_string();
            if key.starts_with("emptymails/INBOX/new/email_") && !key.contains("Zone") {
                latest_inbox_entries.insert(key, e.author());
            }
        }
    }
    
    println!("   Query::all() found {} unique INBOX/new files", all_inbox_entries.len());
    println!("   Query::single_latest_per_key() found {} INBOX/new files", latest_inbox_entries.len());
    
    // Find files that exist in all but not in latest
    let mut missing_from_latest = 0;
    for (key, authors) in &all_inbox_entries {
        if !latest_inbox_entries.contains_key(key) {
            missing_from_latest += 1;
            println!("   File {} has {} versions but missing from latest!", key, authors.len());
        }
    }
    
    if missing_from_latest > 0 {
        println!("\n   WARNING: {} files exist but aren't returned by single_latest_per_key!", missing_from_latest);
        println!("   This might be due to timestamp conflicts or author issues.");
    }
    
    // Let's also check if we're using the wrong query in debug_contents
    println!("\n5. Testing different query filters");
    
    // Try with author filter
    let mut author_filtered = doc.get_many(Query::all().author(self.author)).await?;
    let mut author_count = 0;
    while let Some(_) = author_filtered.next().await {
        author_count += 1;
    }
    println!("   Entries by current author: {}", author_count);
    
    drop(doc);
    println!("\n=== END CONFLICT CHECK ===\n");
    Ok(())
}

/// List ALL entries in the document with full details
pub async fn list_all_entries_detailed(&self) -> Result<()> {
    println!("\n=== COMPREHENSIVE ENTRY LISTING ===");
    
    let doc = self.doc.lock().await;
    
    // Test 1: Count with different query methods
    println!("1. Entry counts by query method:");
    
    let mut count_all = 0;
    let mut entries = doc.get_many(Query::all()).await?;
    while let Some(_) = entries.next().await {
        count_all += 1;
    }
    println!("   Query::all(): {} entries", count_all);
    
    let mut count_latest = 0;
    let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
    while let Some(_) = entries.next().await {
        count_latest += 1;
    }
    println!("   Query::single_latest_per_key(): {} entries", count_latest);
    
    let mut count_author = 0;
    let mut entries = doc.get_many(Query::all().author(self.author)).await?;
    while let Some(_) = entries.next().await {
        count_author += 1;
    }
    println!("   Query::all().author(current): {} entries", count_author);
    
    // Test 2: List files in INBOX/new with all methods
    println!("\n2. INBOX/new files by query method:");
    
    let mut inbox_files_all = HashSet::new();
    let mut entries = doc.get_many(Query::all()).await?;
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key()).to_string();
            if key.starts_with("emptymails/INBOX/new/email_") && !key.contains("Zone") {
                inbox_files_all.insert(key);
            }
        }
    }
    
    let mut inbox_files_latest = HashSet::new();
    let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key()).to_string();
            if key.starts_with("emptymails/INBOX/new/email_") && !key.contains("Zone") {
                inbox_files_latest.insert(key);
            }
        }
    }
    
    println!("   Query::all() found: {} INBOX/new files", inbox_files_all.len());
    println!("   Query::single_latest_per_key() found: {} INBOX/new files", inbox_files_latest.len());
    
    // Show differences
    let only_in_all: Vec<_> = inbox_files_all.difference(&inbox_files_latest).collect();
    if !only_in_all.is_empty() {
        println!("\n   Files found by Query::all() but NOT by single_latest_per_key:");
        for file in only_in_all.iter().take(10) {
            println!("   - {}", file);
        }
        if only_in_all.len() > 10 {
            println!("   ... and {} more", only_in_all.len() - 10);
        }
    }
    
    // Test 3: Check specific missing file with raw key query
    println!("\n3. Direct key lookup for email_369:");
    let test_key = "emptymails/INBOX/new/email_369";
    
    // Try exact key with all entries
    let mut found = false;
    let mut entries = doc.get_many(Query::key_exact(test_key)).await?;
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            found = true;
            println!("   FOUND: author={}, timestamp={:?}", 
                     &e.author().to_string()[..8], e.timestamp());
        }
    }
    
    if !found {
        println!("   NOT FOUND with Query::key_exact()");
        
        // Try prefix search
        let mut entries = doc.get_many(Query::key_prefix("emptymails/INBOX/new/email_369")).await?;
        while let Some(entry) = entries.next().await {
            if let Ok(e) = entry {
                let key = String::from_utf8_lossy(e.key());
                println!("   Found with prefix search: {}", key);
            }
        }
    }
    
    drop(doc);
    println!("\n=== END COMPREHENSIVE LISTING ===\n");
    Ok(())
}

/// Test adding a specific missing file with detailed diagnostics
pub async fn test_add_missing_file(&self, file_name: &str) -> Result<()> {
    println!("\n=== TESTING ADD FOR {} ===", file_name);
    
    let file_path = self.base_path.join("emptymails/INBOX/new").join(file_name);
    let key = format!("emptymails/INBOX/new/{}", file_name);
    
    // Step 1: Check if file exists on disk
    if !file_path.exists() {
        println!("ERROR: File does not exist on disk!");
        return Ok(());
    }
    
    let metadata = fs::metadata(&file_path)?;
    println!("1. File exists on disk: {} bytes", metadata.len());
    
    // Step 2: Read the file
    let content = fs::read(&file_path)?;
    println!("2. File read successfully: {} bytes", content.len());
    
    // Step 3: Check if it already exists in the document
    println!("3. Checking if already in document...");
    let doc = self.doc.lock().await;
    
    let mut existing = doc.get_many(Query::all().key_exact(&key)).await?;
    let mut exists = false;
    while let Some(entry) = existing.next().await {
        if let Ok(e) = entry {
            exists = true;
            println!("   FOUND existing entry: author={}, timestamp={:?}", 
                     &e.author().to_string()[..8], e.timestamp());
        }
    }
    
    if exists {
        println!("   File already exists in document!");
        drop(doc);
        return Ok(());
    }
    
    println!("   File NOT in document, attempting to add...");
    
    // Step 4: Try to add it
    let content_to_store = if content.is_empty() { vec![0u8] } else { content };
    
    match doc.set_bytes(self.author, key.clone(), content_to_store).await {
        Ok(hash) => {
            println!("4. SUCCESS: Added with hash: {}", hash);
        }
        Err(e) => {
            println!("4. ERROR: Failed to add: {}", e);
            drop(doc);
            return Ok(());
        }
    }
    
    // Step 5: Verify it was added
    println!("5. Verifying addition...");
    let mut verify = doc.get_many(Query::all().key_exact(&key)).await?;
    let mut verified = false;
    
    while let Some(entry) = verify.next().await {
        if let Ok(e) = entry {
            verified = true;
            println!("   VERIFIED: Entry exists with author={}", &e.author().to_string()[..8]);
        }
    }
    
    if !verified {
        println!("   ERROR: Entry not found after adding!");
    }
    
    // Step 6: Check with different query methods
    println!("6. Checking with different queries...");
    
    let with_latest = doc.get_many(Query::single_latest_per_key().key_exact(&key)).await?
        .next().await.is_some();
    println!("   Found with single_latest_per_key: {}", with_latest);
    
    let with_author = doc.get_many(Query::all().author(self.author).key_exact(&key)).await?
        .next().await.is_some();
    println!("   Found with author filter: {}", with_author);
    
    drop(doc);
    println!("=== END TEST ===\n");
    Ok(())
}

/// Add all missing files from disk to the document
pub async fn add_missing_files(&self) -> Result<()> {
    println!("\n=== ADDING MISSING FILES ===");
    
    // First, get all files from disk
    let mut disk_files = HashSet::new();
    let mut disk_dirs = HashSet::new();
    let mut zone_count = 0;
    self.scan_disk_recursive(&self.base_path, &mut disk_files, &mut disk_dirs, &mut zone_count)?;
    
    // Get all files from document
    let doc = self.doc.lock().await;
    let mut doc_files = HashSet::new();
    let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
    while let Some(entry) = entries.next().await {
        if let Ok(e) = entry {
            let key = String::from_utf8_lossy(e.key()).to_string();
            if !key.starts_with("d") {
                doc_files.insert(key);
            }
        }
    }
    drop(doc);
    
    // Find missing files
    let missing_files: Vec<String> = disk_files.difference(&doc_files).cloned().collect();
    println!("Found {} missing files", missing_files.len());
    
    // Add each missing file
    let mut success_count = 0;
    let mut fail_count = 0;
    
    for (index, file_key) in missing_files.iter().enumerate() {
        let file_path = self.base_path.join(file_key);
        
        // Progress indicator
        if index % 10 == 0 {
            println!("Progress: {}/{} files processed...", index, missing_files.len());
        }
        
        // Try to add the file
        match self.add_file_to_sync(&file_path).await {
            Ok(()) => {
                success_count += 1;
                info!("Added missing file: {}", file_key);
            }
            Err(e) => {
                fail_count += 1;
                error!("Failed to add {}: {}", file_key, e);
            }
        }
    }
    
    println!("\n=== MISSING FILES ADDITION COMPLETE ===");
    println!("Successfully added: {} files", success_count);
    println!("Failed to add: {} files", fail_count);
    
    // Verify new counts
    let doc = self.doc.lock().await;
    let mut new_count = 0;
    let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
    while let Some(_) = entries.next().await {
        new_count += 1;
    }
    drop(doc);
    
    println!("Total entries in document now: {}", new_count);
    println!("=== END ===\n");
    
    Ok(())
}

/// Alternative: Re-scan a specific directory thoroughly
pub async fn rescan_directory(&self, dir_path: &Path) -> Result<()> {
    println!("\n=== RE-SCANNING DIRECTORY: {:?} ===", dir_path);
    
    let full_path = if dir_path.is_absolute() {
        dir_path.to_path_buf()
    } else {
        self.base_path.join(dir_path)
    };
    
    if !full_path.exists() {
        println!("ERROR: Directory does not exist!");
        return Ok(());
    }
    
    let mut added_count = 0;
    let mut skipped_count = 0;
    let mut error_count = 0;
    
    // Read directory entries
    let entries = fs::read_dir(&full_path)?;
    
    for entry in entries {
        let entry = entry?;
        let path = entry.path();
        
        if entry.metadata()?.is_file() {
            let file_name = entry.file_name().to_string_lossy().to_string();
            
            // Skip Zone.Identifier files
            if file_name.ends_with(":Zone.Identifier") {
                continue;
            }
            
            // Check if already exists
            let relative_path = self.get_relative_path(&path)?;
            let key = relative_path.to_string_lossy().to_string();
            
            if self.entry_exists(&key).await? {
                skipped_count += 1;
                debug!("File already exists: {}", key);
                continue;
            }
            
            // Try to add
            match self.add_file_to_sync(&path).await {
                Ok(()) => {
                    added_count += 1;
                    println!("Added: {}", key);
                }
                Err(e) => {
                    error_count += 1;
                    error!("Failed to add {}: {}", key, e);
                }
            }
        }
    }
    
    println!("\n=== RE-SCAN COMPLETE ===");
    println!("Added: {} files", added_count);
    println!("Skipped (already exist): {} files", skipped_count);
    println!("Errors: {} files", error_count);
    println!("=== END ===\n");
    
    Ok(())
}
pub async fn get_sync_status(&self) -> Result<SyncStatus> {
        let doc = self.doc.lock().await;
        let mut entries = doc.get_many(Query::single_latest_per_key()).await?;
        let mut file_count = 0;
        let mut dir_count = 0;

        while let Some(entry) = entries.next().await {
            if let Ok(e) = entry {
                let key = String::from_utf8_lossy(e.key());
                if key.starts_with("d") {
                    dir_count += 1;
                } else {
                    file_count += 1;
                }
            }
        }
        drop(doc);

        let connected_peers = self.connected_peers.lock().await.len();

        Ok(SyncStatus {
            node_id: self.node_id().to_string(),
            namespace_id: self.namespace_id().await?.to_string(),
            connected_peers,
            total_files: file_count,
            total_directories: dir_count,
        })
    }

    /// List files for web interface
    pub async fn list_files_for_web(&self) -> Result<Vec<FileInfo>> {
        let mut files = Vec::new();
        let doc = self.doc.lock().await;
        let mut entries = doc.get_many(Query::single_latest_per_key()).await?;

        while let Some(entry) = entries.next().await {
            if let Ok(e) = entry {
                let key = String::from_utf8_lossy(e.key()).to_string();

                if key.starts_with("d") {
                    // Directory
                    let dir_path = &key[1..]; // Remove "d" prefix
                    files.push(FileInfo {
                        name: dir_path.split('/').last().unwrap_or(&dir_path).to_string(),
                        size: 0,
                        is_directory: true,
                        path: dir_path.to_string(),
                    });
                } else {
                    // File - skip Zone.Identifier files
                    if key.ends_with(":Zone.Identifier") {
                        continue;
                    }

                    let size = match e.content_bytes(&*doc).await {
                        Ok(content) => {
                            if content.as_ref() == &[0u8] {
                                0 // Empty file placeholder
                            } else {
                                content.len() as u64
                            }
                        }
                        Err(_) => 0,
                    };

                    files.push(FileInfo {
                        name: key.split('/').last().unwrap_or(&key).to_string(),
                        size,
                        is_directory: false,
                        path: key,
                    });
                }
            }
        }

        drop(doc);

        // Sort: directories first, then files, both alphabetically
        files.sort_by(|a, b| {
            match (a.is_directory, b.is_directory) {
                (true, false) => std::cmp::Ordering::Less,
                (false, true) => std::cmp::Ordering::Greater,
                _ => a.name.cmp(&b.name),
            }
        });

        Ok(files)
    }

    /// Download a file for web interface
    pub async fn download_file_for_web(&self, file_path: String) -> Result<impl warp::Reply, warp::Rejection> {
        let doc = self.doc.lock().await;
        let mut entries = doc.get_many(Query::single_latest_per_key().key_exact(&file_path)).await
            .map_err(|_| warp::reject::reject())?;

        if let Some(entry_result) = entries.next().await {
            match entry_result {
                Ok(entry) => {
                    match entry.content_bytes(&*doc).await {
                        Ok(content) => {
                            let actual_content = if content.as_ref() == &[0u8] {
                                // Empty file
                                Vec::new()
                            } else {
                                content.to_vec()
                            };

                            let filename = file_path.split('/').last().unwrap_or("download");
                            
                            Ok(warp::reply::with_header(
                                warp::reply::with_header(
                                    actual_content,
                                    "content-disposition",
                                    format!("attachment; filename=\"{}\"", filename),
                                ),
                                "content-type",
                                "application/octet-stream",
                            ))
                        }
                        Err(_) => Err(warp::reject::reject()),
                    }
                }
                Err(_) => Err(warp::reject::reject()),
            }
        } else {
            Err(warp::reject::reject())
        }
    }

    /// Handle file upload from web interface
    pub async fn handle_file_upload(&self, mut form: warp::multipart::FormData) -> Result<impl warp::Reply, warp::Rejection> {
        let mut uploaded_files = Vec::new();

        // Fix the ambiguous try_next call by using the specific trait
        while let Some(part) = futures_util::TryStreamExt::try_next(&mut form).await.map_err(|_| warp::reject::reject())? {
            if part.name() == "file" {
                let filename = part.filename().unwrap_or("unnamed_file").to_string();
                
                // Skip hidden/system files
                if filename.starts_with('.') || filename.contains(":Zone.Identifier") {
                    continue;
                }

                let stream = part.stream();
                let mut reader = StreamReader::new(
                    stream.map_err(|e| std::io::Error::new(std::io::ErrorKind::Other, e))
                );

                // Read file content
                let mut content = Vec::new();
                tokio::io::AsyncReadExt::read_to_end(&mut reader, &mut content)
                    .await
                    .map_err(|_| warp::reject::reject())?;

                // Save to sync directory
                let file_path = self.base_path.join(&filename);

                // Create parent directories if needed
                if let Some(parent) = file_path.parent() {
                    tokio::fs::create_dir_all(parent)
                        .await
                        .map_err(|_| warp::reject::reject())?;
                }

                tokio::fs::write(&file_path, &content)
                    .await
                    .map_err(|_| warp::reject::reject())?;

                // Add to sync system
                self.add_file_to_sync(&file_path)
                    .await
                    .map_err(|_| warp::reject::reject())?;

                uploaded_files.push(filename);
                info!("Uploaded file via web: {:?}", file_path);
            }
        }

        if uploaded_files.is_empty() {
            return Err(warp::reject::reject());
        }

        Ok(warp::reply::json(&format!("Uploaded {} file(s): {}", 
            uploaded_files.len(), 
            uploaded_files.join(", ")
        )))
    }

    /// Connect to another peer using a ticket
    pub async fn connect_with_ticket(&self, ticket_str: String) -> Result<()> {
        info!("Attempting to connect with ticket: {}", ticket_str);
        
        // Parse the ticket
        let ticket: iroh::docs::DocTicket = ticket_str.parse()
            .context("Failed to parse document ticket")?;
        
        // Import the document (this connects to the peer)
        let _imported_doc = self.client.docs().import(ticket).await
            .context("Failed to import document from ticket")?;
        
        info!("Successfully connected via ticket");
        Ok(())
    }

    /// Generate a new sharing ticket
    pub async fn generate_ticket(&self) -> Result<String> {
        let doc = self.doc.lock().await;
        let ticket = doc.share(ShareMode::Write, AddrInfoOptions::RelayAndAddresses).await?;
        drop(doc);
        Ok(ticket.to_string())
    }
}

/// Start web interface - this is a standalone function, not a method
pub async fn start_web_interface(sync_dir: Arc<Mutex<SyncDir>>, port: u16) -> Result<()> {
    info!("Starting web interface on port {}", port);

    // CORS for mobile browsers
    let cors = warp::cors()
        .allow_any_origin()
        .allow_headers(vec!["content-type", "authorization"])
        .allow_methods(vec!["GET", "POST", "PUT", "DELETE", "OPTIONS"]);

    // Status endpoint
    let sync_dir_status = Arc::clone(&sync_dir);
    let status = warp::path("api")
        .and(warp::path("status"))
        .and(warp::get())
        .and_then(move || {
            let sync_dir = Arc::clone(&sync_dir_status);
            async move {
                match sync_dir.lock().await.get_sync_status().await {
                    Ok(status) => Ok(warp::reply::json(&status)),
                    Err(_) => Err(warp::reject::reject()),
                }
            }
        });

    // File list endpoint
    let sync_dir_files = Arc::clone(&sync_dir);
    let files = warp::path("api")
        .and(warp::path("files"))
        .and(warp::get())
        .and_then(move || {
            let sync_dir = Arc::clone(&sync_dir_files);
            async move {
                match sync_dir.lock().await.list_files_for_web().await {
                    Ok(files) => Ok(warp::reply::json(&files)),
                    Err(_) => Err(warp::reject::reject()),
                }
            }
        });

    // File download endpoint
    let sync_dir_download = Arc::clone(&sync_dir);
    let download = warp::path("api")
        .and(warp::path("download"))
        .and(warp::path::param::<String>())
        .and(warp::get())
        .and_then(move |file_path: String| {
            let sync_dir = Arc::clone(&sync_dir_download);
            async move {
                sync_dir.lock().await.download_file_for_web(file_path).await
            }
        });

    // File upload endpoint
    let sync_dir_upload = Arc::clone(&sync_dir);
    let upload = warp::path("api")
        .and(warp::path("upload"))
        .and(warp::post())
        .and(warp::multipart::form().max_length(100 * 1024 * 1024)) // 100MB limit
        .and_then(move |form: warp::multipart::FormData| {
            let sync_dir = Arc::clone(&sync_dir_upload);
            async move {
                sync_dir.lock().await.handle_file_upload(form).await
            }
        });

    // Connect via ticket endpoint
    let sync_dir_connect = Arc::clone(&sync_dir);
    let connect = warp::path("api")
        .and(warp::path("connect"))
        .and(warp::post())
        .and(warp::body::json())
        .and_then(move |req: ConnectRequest| {
            let sync_dir = Arc::clone(&sync_dir_connect);
            async move {
                match sync_dir.lock().await.connect_with_ticket(req.ticket).await {
                    Ok(_) => Ok(warp::reply::json(&"Connected successfully")),
                    Err(e) => {
                        error!("Connection failed: {}", e);
                        Err(warp::reject::reject())
                    }
                }
            }
        });

    // Generate ticket endpoint  
    let sync_dir_ticket = Arc::clone(&sync_dir);
    let ticket = warp::path("api")
        .and(warp::path("ticket"))
        .and(warp::get())
        .and_then(move || {
            let sync_dir = Arc::clone(&sync_dir_ticket);
            async move {
                match sync_dir.lock().await.generate_ticket().await {
                    Ok(ticket) => Ok(warp::reply::json(&ticket)),
                    Err(_) => Err(warp::reject::reject()),
                }
            }
        });

    // Serve static HTML file
    let index = warp::get()
        .and(warp::path::end())
        .and_then(|| async {
            Ok::<_, warp::Rejection>(warp::reply::html(include_str!("../web/index.html")))
        });

    let routes = status
        .or(files)
        .or(download)
        .or(upload)
        .or(connect)
        .or(ticket)
        .or(index)
        .with(cors);

    println!("\n=== WEB INTERFACE STARTED ===");
    println!("🌐 Local: http://localhost:{}", port);
    println!("📱 Mobile: http://YOUR_PC_IP:{}", port);
    println!("===============================\n");

    warp::serve(routes)
        .run(([0, 0, 0, 0], port))
        .await;

    Ok(())
}

